<?php
/**
 * @author integritty <ict@integritty.cz>
 * @copyright 2022 the integritty s.r.o
 * @license Komerční licence; Licenční podmínky: https://www.integritty.cz/external/prestashop/licencni_podminky_integritty.pdf
 * @category PrestaShop Module
 * E-mail: chci@integritty.cz
 * Websites: www.integritty.cz
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once 'vendor/autoload.php';

class SmartForm16 extends Module
{

    const SMARTFORM_CODE = 'smartform_code';
    const SMARTFORM_ADDRESS = 'smartform_address';
    const SMARTFORM_VALIDATION = 'smartform_validation';
    const SMARTFORM_TRACKING_URL = 'https://smartform.integritty.cz';
    const SMARTFORM_COUNTRIES_CZ = 'smartform_countries_cz';
    const SMARTFORM_COUNTRIES_SK = 'smartform_countries_sk';

    public function __construct()
    {
        $this->name = 'smartform16';
        $this->tab = 'front_office_features';
        $this->version = '1.0.3';
        $this->author = 'integritty';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = ['min' => '1.6', 'max' => _PS_VERSION_];
        $this->bootstrap = true;

        parent::__construct();
        $this->registerHook('displayHeader');
        $this->displayName = $this->l('Smartform');
        $this->description = $this->l('Ensures fast and convenient filling of forms without errors. A convenience that is sure to be appreciated not only by your users. The module helps customers to complete postal addresses for the Czech Republic and Slovakia and verifies the correctness of e-mail addresses.');

        $this->confirmUninstall = $this->l('ATTENTION! Customer mailing address hinting and email address validation will no longer work. Do you really want to uninstall the module?');
    }

    public function install()
    {
        Configuration::updateValue(self::SMARTFORM_CODE, '');
        Configuration::updateValue(self::SMARTFORM_ADDRESS, 0);
        Configuration::updateValue(self::SMARTFORM_VALIDATION, 0);
        Configuration::updateValue(self::SMARTFORM_COUNTRIES_CZ, '');
        Configuration::updateValue(self::SMARTFORM_COUNTRIES_SK, '');
        $this->sendDomain();
        return parent::install()
            && $this->registerHook('displayHeader');
    }

    public function uninstall()
    {
        Configuration::deleteByName(self::SMARTFORM_CODE);
        Configuration::deleteByName(self::SMARTFORM_ADDRESS);
        Configuration::deleteByName(self::SMARTFORM_VALIDATION);
        Configuration::deleteByName(self::SMARTFORM_COUNTRIES_CZ);
        Configuration::deleteByName(self::SMARTFORM_COUNTRIES_SK);
        $this->unregisterHook('displayHeader');
        $this->disableDomain();
        if (parent::uninstall()) {
            return true;
        }
        return false;
    }

    public function sendDomain()
    {
        $domain = Tools::getHttpHost(true) . __PS_BASE_URI__;
        $psVersion = _PS_VERSION_;

        $client = new GuzzleHttp\Client(['defaults' => [
            'verify' => false,
        ]]);

        try {
            $request = $client->createRequest(
                'GET',
                self::SMARTFORM_TRACKING_URL . '/api/save/domain',
                ['json' => ['domain' => $domain, 'version' => $psVersion],
                    'future' => true,
                    'exceptions' => false,
                ]
            );
            $client->send($request)->then();
        } catch (\GuzzleHttp\Exception\ConnectException $e) {
            // do nothing, the timeout exception is intended
        }
    }

    public function disableDomain()
    {
        $domain = Tools::getHttpHost(true) . __PS_BASE_URI__;

        $client = new GuzzleHttp\Client(['defaults' => [
            'verify' => false,
        ]]);

        try {
            $request = $client->createRequest(
                'GET',
                self::SMARTFORM_TRACKING_URL . '/api/disable/domain',
                ['json' => ['domain' => $domain, 'message' => 'uninstalled'],
                    'future' => true,
                    'exceptions' => false,
                ]
            );
            $client->send($request)->then();
        } catch (\GuzzleHttp\Exception\ConnectException $e) {
            // do nothing, the timeout exception is intended
        }
    }

    public function displayForm()
    {
        $defaultLang = (int)Configuration::get('PS_LANG_DEFAULT');

        $fieldsForm[0]['form'] = [
            'legend' => [
                'title' => $this->l('Smartform setting'),
            ],
            'input' => [
                [
                    'type' => 'text',
                    'label' => $this->l('ID customer (clientId)'),
                    'name' => self::SMARTFORM_CODE,
                    'required' => true,
                ],
                [
                    'type' => 'switch',
                    'label' => $this->l('Email validation'),
                    'name' => self::SMARTFORM_VALIDATION,
                    'is_bool' => true,
                    'values' => [
                        [
                            'id' => 'validation_on',
                            'value' => 1,
                        ],
                        [
                            'id' => 'validation_off',
                            'value' => 0,
                        ],
                    ],
                ],
                [
                    'type' => 'switch',
                    'label' => $this->l('Whispering of postal addresses'),
                    'name' => self::SMARTFORM_ADDRESS,
                    'is_bool' => true,
                    'values' => [
                        [
                            'id' => 'address_on',
                            'value' => 1,
                        ],
                        [
                            'id' => 'address_off',
                            'value' => 0,
                        ],
                    ],
                ],
                [
                    'type' => 'checkbox',
                    'label' => $this->l('Countries for whispering postal addresses'),
                    'name' => 'smartform_countries',
                    'values' => [
                        'query' => [
                            [
                                'id' => 'cz',
                                'name' => $this->l('Czech'),
                                'val' => 16,
                            ],
                            [
                                'id' => 'sk',
                                'name' => $this->l('Slovakia'),
                                'val' => 37,
                            ],
                        ],
                        'id' => 'id',
                        'name' => 'name',
                    ],
                ],
            ],
            'submit' => [
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right',
            ],
        ];

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;

        // Language
        $helper->default_form_language = empty($this->context->cookie->employee_form_lang) ? (int)(Configuration::get('PS_LANG_DEFAULT')) : (int)$this->context->cookie->employee_form_lang;
        $helper->allow_employee_form_lang = $defaultLang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;
        $helper->toolbar_scroll = true;
        $helper->submit_action = 'submit' . $this->name;
        $helper->toolbar_btn = [
            'save' => [
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name . '&token=' . Tools::getAdminTokenLite('AdminModules'),
            ],
            'back' => [
                'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list'),
            ],
        ];

        $helper->fields_value[self::SMARTFORM_CODE] = Configuration::get(self::SMARTFORM_CODE);
        $helper->fields_value[self::SMARTFORM_ADDRESS] = (int)Configuration::get(self::SMARTFORM_ADDRESS);
        $helper->fields_value[self::SMARTFORM_VALIDATION] = (int)Configuration::get(self::SMARTFORM_VALIDATION);
        $helper->fields_value[self::SMARTFORM_COUNTRIES_CZ] = Configuration::get(self::SMARTFORM_COUNTRIES_CZ);
        $helper->fields_value[self::SMARTFORM_COUNTRIES_SK] = Configuration::get(self::SMARTFORM_COUNTRIES_SK);

        return $helper->generateForm($fieldsForm);
    }

    public function getContent()
    {
        $output = null;

        if (Tools::isSubmit('submit' . $this->name)) {
            $sm_address = Tools::getValue(self::SMARTFORM_ADDRESS);
            Configuration::updateValue(self::SMARTFORM_ADDRESS, $sm_address);
            $sm_validation = Tools::getValue(self::SMARTFORM_VALIDATION);
            Configuration::updateValue(self::SMARTFORM_VALIDATION, $sm_validation);
            $country_cz = Tools::getValue(self::SMARTFORM_COUNTRIES_CZ);
            Configuration::updateValue(self::SMARTFORM_COUNTRIES_CZ, $country_cz);
            $country_sk = Tools::getValue(self::SMARTFORM_COUNTRIES_SK);
            Configuration::updateValue(self::SMARTFORM_COUNTRIES_SK, $country_sk);
            $sf_code = trim(Tools::getValue(self::SMARTFORM_CODE));

            if (!empty($sf_code)) {
                Configuration::updateValue(self::SMARTFORM_CODE, $sf_code);
                $output .= $this->displayConfirmation($this->l('The settings have been saved'));
            } else {
                $output .= $this->displayError($this->l('Smartform customer ID (clientId) is required'));
            }
        }

        $this->context->controller->addCSS($this->_path . 'views/dist/style.css');

        $defaultLanguage = isset($this->context->language->iso_code) ? $this->context->language->iso_code : 'en';
        $this->context->smarty->assign(['lang' => $defaultLanguage]);

        $contact = $this->display(_PS_MODULE_DIR_ . $this->name, '/views/templates/admin/contact.tpl');
        return $output . $this->displayForm() . $contact;
    }

    public function hookDisplayHeader()
    {
        $validation = (bool)Configuration::get(self::SMARTFORM_VALIDATION);
        $suggestion = (bool)Configuration::get(self::SMARTFORM_ADDRESS);
        $section = $this->context->controller->php_self;

        $countryArray = [Configuration::get(self::SMARTFORM_COUNTRIES_CZ), Configuration::get(self::SMARTFORM_COUNTRIES_SK)];
        $allowedCountries = [];

        foreach ($countryArray as $country) {
            if ($country) {
                $countryObject = new Country($country);
                $allowedCountries[$countryObject->id] = $countryObject->iso_code;
            }
        }

        if (in_array($section, [
                'order', 'address', 'authentication', 'contact', 'order-opc'
            ]) && ($validation || $suggestion)) {
            Media::addJsDef([
                'clientId' => Configuration::get(self::SMARTFORM_CODE),
                'addressSuggestion' => $suggestion,
                'emailValidation' => $validation,
                'allowedCountries' => $allowedCountries,
                'section' => $section,
                'translation' => [
                    'firstServerResponse' => $this->l('We asked the server'),
                    'secondServerResponse' => $this->l('and he doesn\'t know the email.'),
                    'incorrectEmail' => $this->l('Incorrect email address.'),
                    'incorrectDomain' => $this->l('Wrong domain name.'),
                    'undefinedError' => $this->l('It is not certain whether the mailbox exists or not.'),
                ],
            ]);

            $this->context->controller->addJS($this->_path . 'views/dist/bundle.js');
            $this->context->controller->addCSS($this->_path . 'views/dist/style.css');
            return $this->display(_PS_MODULE_DIR_ . $this->name, '/views/templates/front/smart.tpl');
        } else {
            return null;
        }
    }
}
